/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

BookmarkTools.namespace("overlay"); 
 
BookmarkTools.overlay = {

	appendURLBarObserver : function(){
		var urlbar = BookmarkTools.$("urlbar");
		urlbar.setAttribute('autocompletesearch', 'cl-index');
	},
	
	toggleTray : function(item){
		if(item.hasAttribute("checked")){
			BookmarkTools.utils.Prefs.setBool('trayIcon', true);
			BookmarkTools.$("bookmarkTools.tray.menu").setAttribute("checked", true);
			BookmarkTools.$("bookmarkTools.tray.statusBar").setAttribute("checked", true);
			BookmarkTools.$("bookmarkTools.tray.icon").hidden = false;
		}else {
			BookmarkTools.utils.Prefs.setBool('trayIcon', false);
			BookmarkTools.$("bookmarkTools.tray.menu").setAttribute("checked", false);
			BookmarkTools.$("bookmarkTools.tray.statusBar").setAttribute("checked", false);
			BookmarkTools.$("bookmarkTools.tray.icon").hidden = true;
		}
	},
		
	loadPrefs : function(){
		if(BookmarkTools.utils.Prefs.getBool('sidebarOpened'))
			BookmarkTools.overlay.loadSideBar();

		if(BookmarkTools.utils.Prefs.getBool('trayIcon')){
			BookmarkTools.$("bookmarkTools.tray.menu").setAttribute("checked", true);
			BookmarkTools.$("bookmarkTools.tray.statusBar").setAttribute("checked", true);
			BookmarkTools.$("bookmarkTools.tray.icon").hidden = false;
		}
	},
		
	/*
		Every selected result will be marked in 
		the book tab above.
	*/
	findSelectedNode : function(tree){
		var firstCell = tree.childNodes[1].childNodes[tree.currentIndex].childNodes[0].childNodes[0];
		var nodes = BookmarkTools.common.Book.getNode(firstCell.getAttribute("value"));
		
		if(nodes.length > 0){
			var bookmarksView = BookmarkTools.$("bookmarks-view");
			var index = bookmarksView.treeBuilder.getIndexOfResource(nodes[0]);

			if (index != -1) {
				bookmarksView.treeBoxObject.view.selection.clearSelection();
			
				if (!bookmarksView.treeBoxObject.view.selection.isSelected(index))
					bookmarksView.treeBoxObject.view.selection.toggleSelect(index);
				bookmarksView.treeBoxObject.ensureRowIsVisible(index);
			}

		}
	},
		
	loadSideBar : function(){
		var box = BookmarkTools.$("xmlSidebarSeamonkey");
		var splitter = BookmarkTools.$("xmlSidebarSeamonkey-splitter");
		box.hidden = !box.hidden;
		splitter.hidden = !splitter.hidden;
		BookmarkTools.utils.Prefs.setBool('sidebarOpened', !box.hidden);
	},
		
	loadAboutDialog : function(){
		window.open("chrome://bookmarktools/content/bookmarkToolsAbout.xul", "_blank", "chrome, centerscreen, modal");
	},
		
	loadManager : function(){ 
		if(BookmarkTools.globals.lockKey != -1)
			window.open("chrome://bookmarktools/content/bookmarkToolsPaneSeamonkey.xul", "_blank", "chrome, titlebar, toolbar, centerscreen, resizable=no, modal");
	},
		
	/*
		Tries to copy data from the last successfully ended index build.
	*/
	tryCopyIndex : function(){
		if(BookmarkTools.utils.Prefs.getBool('wasUpdated')){
			var buildDir = BookmarkTools.utils.FileIO.getBuildDir();
			var searchDir = BookmarkTools.utils.FileIO.getSearchDir();

			this.update(buildDir, searchDir);
			BookmarkTools.utils.Prefs.setBool('wasUpdated', false);
		}
	},
	
	/*
	  Calls clear() and move().
	*/
	update : function(bdir, sdir){
		BookmarkTools.utils.FileIO.clear(sdir);
		BookmarkTools.utils.FileIO.move(bdir, sdir);
	},
		
	startBrowser : function(){
		try {
			//try to acquire a lock for the index
			BookmarkTools.globals.lockKey = BookmarkTools.indexing.Index.acquire();
			
			if(BookmarkTools.globals.lockKey == -1){
				BookmarkTools.$("bookmarkTools.manager.menu").disabled = true;
				BookmarkTools.$("bookmarkTools.manager.statusBar").disabled = true;
				BookmarkTools.$("bookmarkTools.manager.sidebar").disabled = true;
			}
			
			BookmarkTools.overlay.tryCopyIndex(); 
			BookmarkTools.panel.index.tryBuildAutomatic();
		} catch(e){
			BookmarkTools.reportError(e);
		}
	},
		
	/*
		If automatic indexer is running this function
		stops the process and tries to release the lock
		key.
	*/
	closeBrowser : function(){
		try {
			BookmarkTools.utils.Prefs.setBool('isRunning', false);
		
			if(BookmarkTools.indexing.Index && 
				BookmarkTools.indexing.Index.mode == BookmarkTools.indexing.AUTOMATIC && 
				BookmarkTools.indexing.Index.filter && 
				BookmarkTools.indexing.Index.filter.crawler){
				//abort the crawler
				BookmarkTools.indexing.Index.filter.crawler._abort();
			}
		} catch(e){
			BookmarkTools.reportError(e);
		}
	},
		
	appendListener : function(){
		window.addEventListener("load", this.appendURLBarObserver, false);
		window.addEventListener("load", this.loadPrefs, false);
		window.addEventListener("load", this.startBrowser, false);
		window.addEventListener("unload", this.closeBrowser, false);
	}

};

BookmarkTools.overlay.appendListener();