/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

BookmarkTools.namespace("indexing");

BookmarkTools.indexing.Language = {

	map: null,
	//"en-us": null, "en-gb": null
	supportedLanguages : {
		"de": null, "en": null, "fi": null, "en-us": null, "en-gb": null,
		"fr": null, "it": null, "pt": null, "es": null, "sv": null
	},

	init : function(){
		var file = BookmarkTools.utils.FileIO.newFile(
			"extensions/" + BOOKMARKTOOLS_EXTENSION + "/resources/languages.txt"
		);
		
		this.map = BookmarkTools.utils.FileIO.readFileLineByLine(file, ',');
	},
	
	_init : function(){
		this.map = null;
	},

	isSupported : function(v){
		return this.supportedLanguages[v] != undefined;
	},
	
	convert : function(v){
		v = v.toLowerCase();
		if(v.length > 2){
			//handle language codes like de-DE -> de
			if(v.indexOf('-') == 2){
				v = v.substring(0, 2);
			}else {
				//handle language codes like english -> en
				var shortcut = BookmarkTools.indexing.Language.map.get(v);
				v = (shortcut != undefined && shortcut != "null") ? shortcut : v;
			}
		}
		return v;
	}

};

BookmarkTools.indexing.Stopwords = {

	stopwords : null,
	
	init : function(){
		this.stopwords = [];

		var entries = BookmarkTools.utils.FileIO.listFiles(
			"extensions/" + BOOKMARKTOOLS_EXTENSION + "/resources/stopwords"
		);
		
		var file, language, map;
		
		while(entries.hasMoreElements()){
			file = entries.getNext().QueryInterface(Ci.nsIFile);
			language = file.leafName.replace(/\.txt/gi, "");
			if(BookmarkTools.indexing.Language.isSupported(language)){
				map = BookmarkTools.utils.FileIO.readFileLineByLine(file);
				this.stopwords[language] = map;
			}
		}
	},
	
	_init : function(){
		this.stopwords = null;
	},
	
	contains : function(word, language){
		if(BookmarkTools.utils.ObjectUtils.isNotUndefined(this.stopwords[language])){
			return this.stopwords[language].contains(word);
		}
		return false;
	},
	
	filter : function(tokens, language){
		var filtered = [], token, i, index = 0;
		for(i = 0, l = tokens.length ;i < l; i++){
			token = tokens[i];
			if(!this.contains(token, language)){
				filtered[index++] = token;
			}
		}
		return filtered;
	}

};

//TODO move to jsHelper and write single array choise
BookmarkTools.indexing.QuickSort = {

	ASC : 0,
	DESC : 1,

	compare : function(a, b, type){
		if(type == BookmarkTools.indexing.QuickSort.ASC) return a < b;
		return a > b;
	},

    exch : function(keys, values, i, j) {
        var swapValue = values[i];
        values[i] = values[j];
        values[j] = swapValue;
		var swapKey = keys[i];
        keys[i] = keys[j];
        keys[j] = swapKey;
    },
	
    partition : function(keys, values, left, right, type) {
        var i = left - 1;
        var j = right;
        while (true) {
            while (this.compare(values[++i], values[right], type))      
                ;                               
            while (this.compare(values[right], values[--j], type))      
                if (j == left) break;           
            if (i >= j) break;                 
            this.exch(keys, values, i, j);                      
        }
        this.exch(keys, values, i, right);                      
        return i;
    },

   execute : function(keys, values, left, right, type) {
        if (right <= left) return;
        var i = this.partition(keys, values, left, right, type);
        this.execute(keys, values, left, i - 1, type);
        this.execute(keys, values, i + 1, right, type);
    }
	
};

BookmarkTools.indexing.NGram = {

	profileLength : 300,
	categories : null,
	nGramFile : "ngram.xml",
	
	arSort : function(inputArr) {
		var valArr = [], keyArr = [], populateArr = [];
		var k, i, tmp, index = 0;
		
		// Get key and value arrays
		for (k in inputArr) {
			if (inputArr.hasOwnProperty) {
				tmp = inputArr[k];
				valArr[index] = tmp;
				keyArr[index] = k;
				index++;
			}
		}
		try {
			// Sort our new temporary arrays
			BookmarkTools.indexing.QuickSort.execute(
				keyArr, valArr, 0, keyArr.length - 1, BookmarkTools.indexing.QuickSort.DESC
			);  		
		} catch (e) {
			BookmarkTools.reportError(e);
			return populateArr;
		}
		
		for (i = 0; i < this.profileLength; i++) {
			tmp = keyArr[i];
			populateArr[tmp] = i;
		}

		return populateArr;
	},

	stretch : function(word, n){
		if(n == 1) return word;
	
		var w = '_' + word;
		var i;
		
		for(i = 0, l = n - 1 ;i < l; i++)
			w += '_';
		return w;
	},
	
	outOfPlaceDistance : function(category, newDocument){
		var MAX = this.profileLength;
		var sum = 0, ng, a, b;
	
		for(ng in newDocument){
			a = category[ng];
			b = newDocument[ng];

			if(a == undefined){
				sum += MAX;
			}else {
				sum += Math.abs(a - b);
			}
		}
		return sum;
	},

	getNGrams : function(word, n){
		var ngrams = [];
		var w = this.stretch(word, n);
		var ng, i;
		
		for(i = 0, l = w.length - (n - 1) ;i < l; i++){
			ng = w.substring(i, i + n);
			ngrams[i] = ng;
		}
		return ngrams;
	},
	
	writeTrainingData : function(){
		try {
			var xml = BookmarkTools.utils.XMLUtils.getEmptyXML(
				BookmarkTools.utils.StringUtils.EMPTY,
				BookmarkTools.utils.StringUtils.EMPTY,
				"categories"
			);
			
			var categoriesNode = xml.documentElement;
			for(var language in this.categories){
				var categoryNode = document.createElement("category");
				categoryNode.setAttribute("name", language);
				
				var category = this.categories[language];
				for(var ng in this.categories[language]){
					var ngramNode = document.createElement("ngram");
					ngramNode.setAttribute("name", ng);
					categoryNode.appendChild(ngramNode);
				}
				categoriesNode.appendChild(categoryNode);
			}
			
			var xmlString = BookmarkTools.utils.XMLUtils.toString(xml);
			
			var file = BookmarkTools.utils.FileIO.newFile(
				"extensions/" + BOOKMARKTOOLS_EXTENSION + "/resources/" + this.nGramFile
			);
			
			BookmarkTools.utils.FileIO.writeLineToFile(file, xmlString);
		} catch(e){
			BookmarkTools.reportError(e);
		}
	},
	
	readTrainingData : function(){
		try {
			this.categories = [];
		
			var file = BookmarkTools.utils.FileIO.newFile(
				"extensions/" + BOOKMARKTOOLS_EXTENSION + "/resources/" + this.nGramFile
			);
		
			var xmlString = BookmarkTools.utils.FileIO.readFile(file); 
			var xml = BookmarkTools.utils.XMLUtils.parseXML(xmlString); 

			if(xml != null){
			
				var categoriesNode = xml.documentElement;
				
				for(var i = 0, li = categoriesNode.childNodes.length ;i < li; i++){
					var categoryNode = categoriesNode.childNodes[i];
					var language = categoryNode.getAttribute("name");
					this.categories[language] = [];

					for(var j = 0, lj = categoryNode.childNodes.length ;j < lj; j++){
						var ngramNode = categoryNode.childNodes[j];
						var ng = ngramNode.getAttribute("name");
						this.categories[language][ng] = j;
					}
				}
			}
		} catch(e){
			BookmarkTools.reportError(e);
		}
	},

	addCategory : function(text, language){
		if(this.categories == null)
			this.categories = [];
	
		var words = new BookmarkTools.indexing.WhitespaceTokenizer().tokenize(
			text
		);
		
		var word, ngrams, ng, i, j, n;
		
		for(n = 3 ;n <= 3; n++){
			for(i = 0, li = words.length ;i < li; i++){
				word = words[i];
				ngrams = this.getNGrams(word, n);
				if(this.categories[language] == undefined)
					this.categories[language] = [];
				for(j = 0, lj = ngrams.length ;j < lj; j++){
					ng = ngrams[j];
					if(this.categories[language][ng] == undefined)
						this.categories[language][ng] = 0;
					this.categories[language][ng]++;
				}
			}
		}
		
		this.categories[language] = this.arSort(this.categories[language]);
	},

	classifyDocument : function(words){
		var newDocument = [];
		var word, ngrams, ng, i, j, n;
		
		for(n = 3 ;n <= 3; n++){
			for(i = 0, li = words.length ;i < li; i++){
				word = words[i];
				ngrams = this.getNGrams(word, n);
				for(j = 0, lj = ngrams.length ;j < lj; j++){
					ng = ngrams[j];
					if(newDocument[ng] == undefined)
						newDocument[ng] = 0;
					newDocument[ng]++;
				}
			}
		}
		newDocument = this.arSort(newDocument);
		
		//TODO use 'profileLength' instead of MAX_VALUE
		var minDistance = Number.MAX_VALUE;
		var minDistanceLanguage = "";
		var lang;
		
		for(lang in this.categories){
			var category = this.categories[lang];
			var distance = this.outOfPlaceDistance(category, newDocument, this.profileLength);

			if(distance < minDistance){
				minDistance = distance;
				minDistanceLanguage = lang;
			}
		}

		return minDistanceLanguage;
	}
	
};